/*
File: Timeout.java
Date		Author		Changes
Jan 23 98	Lukasz Beca	Created
*/

package webwisdom.tango.threads;

import java.util.*;
import webwisdom.tango.threads.*;
import webwisdom.tango.protocols.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.consts.*;

public class Timeout implements Runnable {

	private Hashtable _pingtable;
	private Thread _th;
	private ComThread _ct;
	private int _interval;
	
	public Timeout(int time, ComThread ct) {
	
		_pingtable = new Hashtable();
		_ct = ct;
		_interval = time;
	  _th = new Thread(this);
    _th.start();
	}
	
	public void userResponded(int UID) {
	
		StateEntry se;
		
		System.out.println("Timeout: response from user " + UID + " received");
		
		se = (StateEntry) _pingtable.get(new Integer(UID));
		se.setState(Const.OK);
	
	}
	
	public void userRemoved(int UID) {
	
		_pingtable.remove(new Integer(UID));
	
	}
	
	public void newUser(int UID, ServerProtocol in) {
	
		StateEntry se = new StateEntry(in);
		
		System.out.println("Timeout: new user added to timeout");
		
		_pingtable.put(new Integer(UID), se);
	}
			
	public void run() {
	
		Enumeration elems, keys;
		PingMessage pm = new PingMessage();
		StateEntry se;
		int UID;
		
		System.out.println("Timeout: timeout thread started");
		
		while(1 == 1) {
		
			try {
				
				System.out.println("Timeout: going to sleep");
			
				Thread.sleep(_interval);
				
				System.out.println("Timeout: waking up");
				
			} catch(Exception e) {
			
				e.printStackTrace();
			
			}
		
			elems = _pingtable.elements();
			keys = _pingtable.keys();
		
			while(elems.hasMoreElements()) {
			
				se = (StateEntry) elems.nextElement();
				UID = ((Integer) keys.nextElement()).intValue();
			
				if (se.getState() == Const.OK) {
				
					_ct.sendTo(UID, pm);
					se.setState(Const.TESTING);
				
				} else {
				
					System.out.println("Timeout: user " + UID + " crashed");
				
					se.deleteUser();
				
				}						
			}								
		}
	}
}

class StateEntry {
	
	private ServerProtocol _in;
	private byte _state;

	public StateEntry(ServerProtocol in) {
	
		_state = Const.OK;
		_in = in;
	
	}
	
	public int getState() {
	
		return _state;
	
	}
	
	public void setState(byte state) {
	
		_state = state;
	}

	public void deleteUser() {
	
		_in.logout();
		_in.stop();
	
	}
}

